@@ -1,4 +1,4 @@
-#
+
 #    FormValidator.pm - Object that validates form input data.
 #
 #    This file is part of Data::FormValidator.
@@ -9,7 +9,7 @@
 #    Copyright (C) 1999 Francis J. Lacoste, iNsu Innovations
 #    Parts Copyright 1996-1999 by Michael J. Heins 
 #    Parts Copyright 1996-1999 by Bruce Albrecht  
-#    Parts Copyright 2001-2003 by Mark Stosberg 
+#    Parts Copyright 2001-2005 by Mark Stosberg 
 #
 #    Parts of this module are based on work by
 #    Bruce Albrecht,  contributed to
@@ -31,7 +31,7 @@ use Data::FormValidator::Constraints (qw/:validators :matchers/);
 
 use vars qw( $VERSION $AUTOLOAD @ISA @EXPORT_OK %EXPORT_TAGS );
 
-$VERSION = '4.00_02';
+$VERSION = '4.21_01';
 
 require Exporter;
 @ISA = qw(Exporter);
@@ -167,9 +167,9 @@ sub new {
 
  my $results = Data::FormValidator->check(\%input_hash, \%dfv_profile);
 
-C<check> is the recommended method to use to validate forms. It returns it's results as a
+C<check> is the recommended method to use to validate forms. It returns its results as a
 L<Data::FormValidator::Results|Data::FormValidator::Results> object.  A
-deprecated method C<validate> is also available, returning it's results as an
+deprecated method C<validate> is also available, returning its results as an
 array described below.
 
  use Data::FormValidator;
@@ -185,6 +185,47 @@ multiple values for a single key should be presented as an array reference.
 
 The second argument is a reference to the profile you are validating.
 
+=head2 validate()
+
+    my( $valids, $missings, $invalids, $unknowns ) = 
+        Data::FormValidator->validate( \%input_hash, \%dfv_profile);
+
+C<validate()> provides a deprecated alternative to C<check()>. It has the same input
+syntax, but returns a four element array, described as follows
+
+=over
+
+=item valids
+
+This is a hash reference to the valid fields which were submitted in
+the data. The data may have been modified by the various filters specified.
+
+=item missings
+
+This is a reference to an array which contains the name of the missing
+fields. Those are the fields that the user forget to fill or filled
+with spaces. These fields may comes from the I<required> list or the
+I<dependencies> list.
+
+=item invalids
+
+This is a reference to an array which contains the name of the fields which
+failed one or more of their constraint checks. If there are no invalid fields,
+an empty arrayref will be returned. 
+
+Fields defined with multiple constraints will have an array ref returned in the
+@invalids array instead of a string. The first element in this array is the
+name of the field, and the remaining fields are the names of the failed
+constraints. 
+
+=item unknowns
+
+This is a list of fields which are unknown to the profile. Whether or
+not this indicates an error in the user input is application
+dependant.
+
+=back
+
 =head2 new()
 
 Using C<new()> is only needed for advanced usage, including these cases:
@@ -198,7 +239,7 @@ want by name later with C<check()>. Here's an example:
 
  my $dfv = Data::FormValidator->new({
     profile_1 => { # usual profile definition here },
-    profile_1 => { # another profile definition },
+    profile_2 => { # another profile definition },
  });
 
 
@@ -270,7 +311,7 @@ sub validate {
 
     my $valid   = $data_set->valid();
     my $missing = $data_set->missing();
-    my $invalid = $data_set->{validate_invalid};
+    my $invalid = $data_set->{validate_invalid} || [];
     my $unknown = [ $data_set->unknown ];
 
     return ( $valid, $missing, $invalid, $unknown );
@@ -320,7 +361,7 @@ keys.
 Here is a very simple input profile. Examples of more advanced options are
 described below.
 
-    use Data::FormValidator::Constraints qw(:all);
+    use Data::FormValidator::Constraints qw(:closures);
 
     my $profile = {
         optional => [qw( company
@@ -354,8 +395,8 @@ spaces will be reported as missing.
 
  required_regexp => qr/city|state|zipcode/,
 
-This is a regular expression used to specify additional fields which are
-required.
+This is a regular expression used to specify additional field names for which values
+will be required.
 
 =head2 require_some
 
@@ -432,25 +473,28 @@ the values are references to arrays of the field names in each group.
 This is a hash reference where keys are field names and 
 values are defaults to use if input for the field is missing. 
 
+The values can be code refs which will be used to calculate the 
+value if needed. These code refs will be passed in the DFV::Results
+object as the only parameter. 
+
 The defaults are set shortly before the constraints are applied, and
 will be returned with the other valid data.
 
-
 =head2 filters
 
  # trim leading and trailing whitespace on all fields
  filters       => ['trim'],
 
-This is a reference to an array of filters that will be applied to ALL
-optional and required fields. 
+This is a reference to an array of filters that will be applied to ALL optional
+and required fields, B<before> any constraints are applied. 
 
 This can be the name of a built-in filter
 (trim,digit,etc) or an anonymous subroutine which should take one parameter, 
 the field value and return the (possibly) modified value.
 
-Filters modify the data, so use them carefully. 
+Filters modify the data returned through the results object, so use them carefully. 
 
-See Data::FormValidator::Filters for details on the built-in filters.
+See L<Data::FormValidator::Filters> for details on the built-in filters.
 
 =head2 field_filters
 
@@ -458,10 +502,13 @@ See Data::FormValidator::Filters for details on the built-in filters.
      cc_no => ['digit'],
  },
 
-A hash ref with field names and keys. Values are array references
-of field-specific filters to apply.
+A hash ref with field names as keys. Values are array references of built-in
+filters to apply (trim,digit,etc) or an anonymous subroutine which should take
+one parameter, the field value and return the (possibly) modified value.
 
-See Data::FormValidator::Filters for details on the built-in filters.
+Filters are applied B<before> any constraints are applied. 
+
+See L<Data::FormValidator::Filters> for details on the built-in filters.
 
 =head2 field_filter_regexp_map
 
@@ -477,7 +524,7 @@ match a regular expression.
 
 =head2 constraint_methods
 
- use Data::FormValidator::Constraints (qw:all);
+ use Data::FormValidator::Constraints qw(:closures);
 
  constraint_methods => {
     cc_no      => cc_number({fields => ['cc_type']}),
@@ -516,8 +563,8 @@ B<Example>:
 
  my_zipcode_field   => qr/^\d{5}$/, # match exactly 5 digits
 
-If this field is named in the C<untaint_constraint_fields>, or 
-C<untaint_all_constraints> is effective, be aware of the following: If you
+If this field is named in C<untaint_constraint_fields> or C<untaint_regexp_map>,
+or C<untaint_all_constraints> is effective, be aware of the following: If you
 write your own regular expressions and only match part of the string then
 you'll only get part of the string in the valid hash. It is a good idea to
 write you own constraints like /^regex$/. That way you match the whole string.
@@ -529,8 +576,8 @@ a subroutine reference, to supply custom code
 This will check the input and return true or false depending on the input's validity.
 By default, the constraint function recieves a L<Data::FormValidator::Results>
 object as its first argument, and the value to be validated as the second.  To
-validate a field based more inputs than just the field itself, see C<VALIDATING
-INPUT BASED ON MULTIPLE FIELDS>.
+validate a field based more inputs than just the field itself, see 
+L<VALIDATING INPUT BASED ON MULTIPLE FIELDS>.
 
 B<Examples>:
 
@@ -563,7 +610,7 @@ For more details see L<VALIDATING INPUT BASED ON MULTIPLE FIELDS>.
 
 =head2 constraint_method_regexp_map
 
- use Data::FormValidator::Constraints qw(:all);
+ use Data::FormValidator::Constraints qw(:closures);
 
  # In your profile. 
  constraint_method_regexp_map => {
@@ -590,7 +637,7 @@ may not provide untainting.
 See L<WRITING YOUR OWN CONSTRAINT ROUTINES> in the Data::FormValidator::Constraints
 documention for more information.
 
-This is overridden by C<untaint_constraint_fields>
+This is overridden by C<untaint_constraint_fields> and C<untaint_regexp_map>.
 
 =head2 untaint_constraint_fields
 
@@ -602,6 +649,20 @@ field names. The untainted data will be returned in the valid hash.
 
 This overrides the untaint_all_constraints flag.
 
+=head2 untaint_regexp_map
+
+ untaint_regexp_map => [qr/some_field_\d/],
+
+Specifies that certain fields will be untained if they pass their constraints
+and match one of the regular expressions supplied. This can be set to a single
+regex, or an array reference of regexes. The untainted data will be returned
+in the valid hash.
+
+The above example would untaint the fields named C<some_field_1>, and C<some_field_2>
+but not C<some_field>. 
+
+This overrides the untaint_all_constraints flag.
+
 =head2 missing_optional_valid
 
  missing_optional_valid => 1
@@ -688,6 +749,24 @@ as providing custom messages per field, and handling multiple constraints:
 The hash that's prepared can be retrieved through the C<msgs> method
 described in the L<Data::FormValidator::Results> documentation.
 
+=head2 msgs - callback 
+
+I<This is a new feature. While it expected to be forward-compatible, it hasn't
+yet received the testing the rest of the API has.>   
+
+If the built-in message generation doesn't suit you, it is also possible to
+provide your own by specifying a code reference:
+
+ msgs  =>  \&my_msgs_callback
+
+This will be called as a L<Data::FormValidator::Results> method.  It may
+receive as arguments an additional hash reference of control parameters,
+corresponding to the key names in the usually used in the C<msgs> area of the
+profile. You can ignore this information if you'd like. 
+
+If you have an alternative error message handler you'd like to share, 
+stick in the C<Data::FormValidator::ErrMsgs> and upload it to CPAN. 
+
 =head2 debug
 
 This method is used to print details about what is going on to STDERR.
@@ -724,17 +803,20 @@ Deprecated, but supported
 
 You can pass more than one value into a constraint routine.  For that, the
 value of the constraint should be a hash reference. If you are creating your
-own routines, be sure to read the section labeled L<WRITING YOUR OWN VALIDATION
-ROUTINES>, in the Data::FormValidator::Constraints documentation.  It describes
+own routines, be sure to read the section labeled 
+L<WRITING YOUR OWN CONSTRAINT ROUTINES>, 
+in the Data::FormValidator::Constraints documentation.  It describes
 a newer and more flexible syntax. 
 
 Using the original syntax, one key should be named C<constraint> and should
 have a value set to the reference of the subroutine or the name of a built-in
 validator.  Another required key is C<params>. The value of the C<params> key
 is a reference to an array of the other elements to use in the validation. If
-the element is a scalar, it is assumed to be a field name. If the value is a
-reference, the reference is passed directly to the routine. Don't forget to
-include the name of the field to check in that list, if you are using this syntax.
+the element is a scalar, it is assumed to be a field name. The field is known
+to Data::FormValidator, the value will be filtered through any defined filteres
+before it is passed in.  If the value is a reference, the reference is passed
+directly to the routine.  Don't forget to include the name of the field to
+check in that list, if you are using this syntax.
 
 B<Example>:
 
@@ -821,6 +903,7 @@ sub _check_profile_syntax {
         untaint_all_constraints=> undef,
         validator_packages=> undef,
         untaint_constraint_fields=> undef,
+        untaint_regexp_map => undef,
         debug=> undef,
     );
 
@@ -1006,7 +1089,7 @@ The C<params> value is a reference to an array of the parameters to pass
 to the constraint method. 
 If an element of the C<params> list is a scalar, it is assumed to be naming
 a key of the %input_hash and that value is passed to the routine. 
-If the parameter is a '\' reference, then it is treated literally and passed 
+If the parameter is a reference, then it is treated literally and passed 
 unchanged to the routine.
 
 If you are using the older C<constraint> over 
@@ -1097,11 +1180,29 @@ Albrecht to the MiniVend program.
 
 =head1 BUGS
 
+Bug reports and patches are welcome. Reports which include a failing Test::More
+style test are helpful will receive priority. 
+
 L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Data-FormValidator> 
 
+=head1 CONTRIBUTING
+
+This project is managed using the darcs source control system (
+http://www.darcs.net/ ). My darcs archive is here:
+http://mark.stosberg.com/darcs_hive/dfv/
+
+B<Support Mailing List>
+ 
+If you have any questions, comments, or feature suggestions, post them to the
+support mailing list!  To join the mailing list, visit
+
+L<http://lists.sourceforge.net/lists/listinfo/cascade-dataform>
+
+Messages about DFV sent directly to the maintainer may be redirected here. 
+
 =head1 AUTHOR
 
-Parts Copyright 2001-2003 by Mark Stosberg <markstos@cpan.org>, (Current Maintainer)
+Parts Copyright 2001-2006 by Mark Stosberg <mark at summersault.com>, (Current Maintainer)
 
 Copyright (c) 1999 Francis J. Lacoste and iNsu Innovations Inc.  All rights reserved.
 (Original Author)
@@ -1110,12 +1211,6 @@ Parts Copyright 1996-1999 by Michael J. Heins <mike@heins.net>
 
 Parts Copyright 1996-1999 by Bruce Albrecht  <bruce.albrecht@seag.fingerhut.com>
 
-B<Support Mailing List>
- 
-If you have any questions, comments, bug reports or feature suggestions,
-post them to the support mailing list!  To join the mailing list, visit 
-L<http://lists.sourceforge.net/lists/listinfo/cascade-dataform>
-
 =head1 LICENSE 
 
 This program is free software; you can redistribute it and/or modify
