@@ -15,12 +15,16 @@ use strict;
 
 package Data::FormValidator::Results;
 
+use Carp;
 use Symbol;
 use Data::FormValidator::Filters qw/:filters/;
 use Data::FormValidator::Constraints (qw/:validators :matchers/);
 use vars qw/$AUTOLOAD $VERSION/;
+use overload
+  'bool' => \&_bool_overload_based_on_success,
+  fallback => 1;
 
-$VERSION = 4.00_02;
+$VERSION = 4.21_01;
 
 =pod
 
@@ -42,7 +46,7 @@ Data::FormValidator::Results - results of form input validation.
     # Print the name of invalid fields
     if ( $results->has_invalid ) {
 	foreach my $f ( $results->invalid ) {
-	    print $f, " is invalid: ", $results->invalid( $f ) \n";
+	    print $f, " is invalid: ", $results->invalid( $f ), "\n";
 	}
     }
 
@@ -55,7 +59,7 @@ Data::FormValidator::Results - results of form input validation.
 
     # Print valid fields
     foreach my $f ( $results->valid() ) {
-	print $f, " =  ", $result->valid( $f ), "\n";
+        print $f, " =  ", $results->valid( $f ), "\n";
     }
 
 =head1 DESCRIPTION
@@ -82,7 +86,7 @@ sub _process {
 
  	# Copy data and assumes that all is valid to start with
  		
-	my %data        = $self->_get_data($data);
+	my %data        = $self->_get_input_as_hash($data);
     my %valid	    = %data;
     my @missings    = ();
     my @unknown	    = ();
@@ -95,6 +99,7 @@ sub _process {
     # import valid_* subs from requested packages
 	foreach my $package (_arrayify($profile->{validator_packages})) {
 		if ( !exists $imported_validators{$package} ) {
+			local $SIG{__DIE__}  = \&confess;
 			eval "require $package";
 			if ($@) {
 				die "Couldn't load validator package '$package': $@";
@@ -195,7 +200,7 @@ sub _process {
 		if (ref $valid{$field}) {
 			if ( ref $valid{$field} eq 'ARRAY' ) {
 				for (my $i = 0; $i < scalar @{ $valid{$field} }; $i++) {
-					$valid{$field}->[$i] = undef unless (defined $valid{$field}->[$i] and length $valid{$field}->[$i]);
+					$valid{$field}->[$i] = undef unless (defined $valid{$field}->[$i] and length $valid{$field}->[$i] and $valid{$field}->[$i] !~ /^\x00$/);
 			    }	
                 # If all fields are empty, we delete it.
                 delete $valid{$field} unless grep { defined $_ } @{$valid{$field}};
@@ -203,13 +208,13 @@ sub _process {
 			}
 		}
 		else {
-			delete $valid{$field} unless (defined $valid{$field} and length $valid{$field});
+			delete $valid{$field} unless (defined $valid{$field} and length $valid{$field} and $valid{$field} !~ /^\x00$/);
 		}
 	}
 
     # Check if the presence of some fields makes other optional fields required.
     while ( my ( $field, $deps) = each %{$profile->{dependencies}} ) {
-        if ($valid{$field}) {
+        if (defined $valid{$field}) {
 			if (ref($deps) eq 'HASH') {
 				foreach my $key (keys %$deps) {
                     # Handle case of a key with a single value given as an arrayref
@@ -258,8 +263,14 @@ sub _process {
 	}
 
     # Fill defaults
-	while ( my ($field,$value) = each %{$profile->{defaults}} ) {
-		$valid{$field} = $value unless exists $valid{$field};
+    while ( my ($field,$value) = each %{$profile->{defaults}} ) {
+        unless(exists $valid{$field}) {
+            if (ref($value) && ref($value) eq "CODE") {
+                $valid{$field} = $value->($self);
+            } else {
+                $valid{$field} = $value;
+            }
+        }
 	}
 
     # Check for required fields
@@ -279,23 +290,6 @@ sub _process {
 		push @missings, $field unless ($enough_required_fields >= $num_fields_to_require);
 	}
 
-    #Decide which fields to untaint
-    my ($untaint_all, %untaint_hash);
-	if (defined($profile->{untaint_constraint_fields})) {
-		if (ref $profile->{untaint_constraint_fields} eq "ARRAY") {
-			foreach my $field (@{$profile->{untaint_constraint_fields}}) {
-				$untaint_hash{$field} = 1;
-			}
-		}
-		elsif ($valid{$profile->{untaint_constraint_fields}}) {
-			$untaint_hash{$profile->{untaint_constraint_fields}} = 1;
-		}
-	}
-    elsif ((defined($profile->{untaint_all_constraints}))
-	   && ($profile->{untaint_all_constraints} == 1)) {
-	   $untaint_all = 1;
-    }
-
     # add in the constraints from the regexp maps
     # We don't want to modify the profile, so we use a new variable.
 	$profile->{constraints} ||= {};
@@ -309,6 +303,45 @@ sub _process {
 		_add_constraints_from_map($profile,'constraint_method',\%valid),
 	};
 
+    #Decide which fields to untaint
+    my ($untaint_all, %untaint_hash);
+    if (defined $profile->{untaint_regexp_map} or defined $profile->{untaint_constraint_fields} ) {
+        # first deal with untaint_constraint_fields
+        if (defined($profile->{untaint_constraint_fields})) {
+            if (ref $profile->{untaint_constraint_fields} eq "ARRAY") {
+                foreach my $field (@{$profile->{untaint_constraint_fields}}) {
+                    $untaint_hash{$field} = 1;
+                }
+            }
+            elsif ($valid{$profile->{untaint_constraint_fields}}) {
+                $untaint_hash{$profile->{untaint_constraint_fields}} = 1;
+            }
+        }
+
+        # now look at untaint_regexp_map
+        if(defined($profile->{untaint_regexp_map})) {
+            my @untaint_regexes;
+            if(ref $profile->{untaint_regexp_map} eq "ARRAY") {
+                @untaint_regexes = @{$profile->{untaint_regexp_map}};
+            }
+            else {
+                push(@untaint_regexes, $profile->{untaint_regexp_map});
+            }
+
+            foreach my $regex (@untaint_regexes) {
+                # look at both constraints and constraint_methods
+                foreach my $field (keys %$private_constraints, keys %$private_constraint_methods) {
+                    next if($untaint_hash{$field}); 
+                    $untaint_hash{$field} = 1 if( $field =~ $regex );
+                }
+            }
+        }
+    }
+    elsif ((defined($profile->{untaint_all_constraints}))
+	   && ($profile->{untaint_all_constraints} == 1)) {
+	   $untaint_all = 1;
+    }
+
 	$self->_check_constraints($private_constraints,\%valid,$untaint_all,\%untaint_hash);
 
 	my $force_method_p = 1;
@@ -342,6 +375,12 @@ sub _process {
 This method returns true if there were no invalid or missing fields,
 else it returns false.
 
+As a shorthand, When the $results object is used in boolean context, it is overloaded
+to use the value of success() instead. That allows creation of a syntax like this one used
+in C<CGI::Application::Plugin::ValidateRM>:
+
+ my $results = $self->check_rm('form_display','_form_profile') || return $self->dfv_error_page;
+
 =cut
 
 sub success {
@@ -503,11 +542,17 @@ This method returns a hash reference to error messages. The exact format
 is determined by parameters in the C<msgs> area of the validation profile,
 described in the L<Data::FormValidator> documentation.
 
-This method takes one possible parameter, a hash reference containing the same 
-options that you can define in the validation profile. This allows you to separate
-the controls for message display from the rest of the profile. While validation profiles
-may be different for every form, you may wish to format messages the same way
-across many projects.
+B<NOTE:> the C<msgs> parameter in the profile can take a code reference as a
+value, allowing complete control of how messages are generated. If such a code
+reference was provided there, it will be called here instead of the usual
+processing, described below. It will receive as arguments the L<Data::FormValidator::Results>
+object and a hash reference of control parameters.
+
+The hashref passed in should contain the same options that you can define in
+the validation profile. This allows you to separate the controls for message
+display from the rest of the profile. While validation profiles may be
+different for every form, you may wish to format messages the same way across
+many projects.
 
 Controls passed into the <msgs> method will be applied first, followed by ones
 applied in the profile. This allows you to keep the controls you pass to
@@ -516,6 +561,17 @@ C<msgs> as "global" and override them in a specific profile if needed.
 =cut
 
 sub msgs {
+  my $self = shift;
+  my $msgs = $self->{profile}{msgs} || {};
+  if ((ref $msgs eq 'CODE')) {
+    return $msgs->($self,@_);
+  } else {
+    return $self->_generate_msgs(@_);
+  }
+}
+
+
+sub _generate_msgs {
 	my $self = shift;
 	my $controls = shift || {};
 	if (defined $controls and ref $controls ne 'HASH') {
@@ -574,7 +630,9 @@ sub msgs {
 
 	my $msgs_ref = prefix_hash($profile{prefix},\%msgs);
 
-	$msgs_ref->{ $profile{any_errors} } = 1 if defined $profile{any_errors};
+    if (! $self->success) {
+    	$msgs_ref->{ $profile{any_errors} } = 1 if defined $profile{any_errors};
+    }
 
 	return $msgs_ref;
 
@@ -642,7 +700,14 @@ sub meta {
 
 sub get_input_data {
 	my $self = shift;
-	return $self->{__INPUT_DATA};
+    my %p = @_;
+    if ($p{as_hashref}) {
+        my %hash = $self->_get_input_as_hash( $self->{__INPUT_DATA} );
+        return \%hash;
+    }
+    else {
+	    return $self->{__INPUT_DATA};
+    }
 }
 
 sub get_current_constraint_field {
@@ -660,6 +725,14 @@ sub get_current_constraint_name {
 	return $self->{__CURRENT_CONSTRAINT_NAME};
 }
 
+sub untainted_constraint_value {
+    my $self = shift;
+    my $match = shift;
+
+    return undef unless defined $match;
+    return $self->{__UNTAINT_THIS} ? $match : length $match;
+}
+
 sub set_current_constraint_name {
 	my $self = shift;
 	my $value = shift;
@@ -694,12 +767,14 @@ sub prefix_hash {
 sub _create_sub_from_RE {
 	my $re = shift || return undef;
 	my $untaint_this = shift;
+    my $force_method_p = shift;
 
 	my $sub;
 	# If it's "qr" style
 	if (substr($re,0,1) eq '(') {
 		$sub = sub { 
-            my $val = shift;
+            # With methods, the value is the second argument
+            my $val = $force_method_p ? $_[1] : $_[0];
 			my ($match) = ($val =~ $re); 
 			if ($untaint_this && defined $match) {
                 # pass the value through a RE that matches anything to untaint it.
@@ -713,15 +788,22 @@ sub _create_sub_from_RE {
 
 	}
 	else {
+        local $SIG{__DIE__}  = \&confess;
         my $return_code = ($untaint_this) ? '; return ($& =~ m/(.*)/s)[0] if defined($`);' : '';
-		$sub = eval 'sub { $_[0] =~ '.$re.$return_code. '}';
+        # With methods, the value is the second argument
+        if ($force_method_p) {
+            $sub = eval 'sub { $_[1] =~ '.$re.$return_code. '}';
+        }
+        else {
+            $sub = eval 'sub { $_[0] =~ '.$re.$return_code. '}';
+        }
 	    die "Error compiling regular expression $re: $@" if $@;
 	}
 	return $sub;
 }
 
 
-sub _error_msg_fmt ($$) {
+sub _error_msg_fmt  {
 	my ($fmt,$msg) = @_;
 	$fmt ||= 
 			'<span style="color:red;font-weight:bold"><span class="dfv_errors">* %s</span></span>';
@@ -769,8 +851,9 @@ sub _filter_apply {
 # $constraint_href = $self->_constraint_hash_build($spec,$untaint_p)
 #
 # Input:
-#   - $spec     # Any constraint valid in the profile
-#   - $untaint  # bool for whether we could try to untaint the field. 
+#   - $spec           # Any constraint valid in the profile
+#   - $untaint        # bool for whether we could try to untaint the field. 
+#   - $force_method_p # bool for if it's  a method ?
 #
 # Output:
 #  - $constraint_hashref
@@ -781,8 +864,8 @@ sub _filter_apply {
 # 		is_method  - bool for whether this was a 'constraint' or 'constraint_method'
 
 sub _constraint_hash_build {
-	my ($self,$constraint_spec,$untaint_this) = @_;
-	die "_constraint_hash_build received wrong number of arguments" unless (scalar @_ == 3);
+	my ($self,$constraint_spec,$untaint_this,$force_method_p) = @_;
+	die "_constraint_hash_build received wrong number of arguments" unless (scalar @_ == 4);
 
 	my	$c = {
 			name 		=> $constraint_spec,
@@ -800,7 +883,7 @@ sub _constraint_hash_build {
 	# Check for regexp constraint
 	if ((ref $c->{constraint} eq 'Regexp')
 			or ( $c->{constraint} =~ m@^\s*(/.+/|m(.).+\2)[cgimosx]*\s*$@ )) {
-		$c->{constraint} = _create_sub_from_RE($c->{constraint},$untaint_this);
+		$c->{constraint} = _create_sub_from_RE($c->{constraint},$untaint_this,$force_method_p);
 	}
 	# check for code ref
 	elsif (ref $c->{constraint} eq 'CODE') {
@@ -819,6 +902,7 @@ sub _constraint_hash_build {
 			}
 			# If the constraint name starts with RE_, try looking for it in the Regexp::Common package
 			elsif ($c->{constraint} =~ m/^RE_/) {
+				local $SIG{__DIE__}  = \&confess;
 				$c->{is_method} = 1;
 				$c->{constraint} = eval 'sub { &_create_regexp_common_constraint(@_)}' 
 					|| die "could not create Regexp::Common constraint: $@";
@@ -830,6 +914,7 @@ sub _constraint_hash_build {
 			# try to use match_* first
 			my $routine = 'match_'.$c->{constraint};			
 			if (defined *{qualify_to_ref($routine)}{CODE}) {
+				local $SIG{__DIE__}  = \&confess;
 				$c->{constraint} = eval 'sub { no strict qw/refs/; return defined &{"match_'.$c->{constraint}.'"}(@_)}';
 			}
 			# match_* doesn't exist; if it is supposed to be from the
@@ -839,6 +924,7 @@ sub _constraint_hash_build {
 			}
 			# Load it from Regexp::Common 
 			elsif ($c->{constraint} =~ m/^RE_/) {
+				local $SIG{__DIE__}  = \&confess;
 				$c->{is_method} = 1;
 				$c->{constraint} = eval 'sub { return defined &_create_regexp_common_constraint(@_)}' ||
 					die "could not create Regexp::Common constraint: $@";
@@ -904,6 +990,10 @@ sub _constraint_check_match {
 	my 	($self,$c,$params,$untaint_this) = @_;
 	die "_constraint_check_match received wrong number of arguments" unless (scalar @_ == 4);
 
+    # Store whether or not we want untainting in the object so that constraints
+    # can do the right thing conditionally.
+    $self->{__UNTAINT_THIS} = $untaint_this;
+
     my $match = $c->{constraint}->( @$params );
 
     # We need to make this distinction when untainting,
@@ -925,7 +1015,7 @@ sub _constraint_check_match {
 }
 
 # Figure out whether the data is a hash reference of a param-capable object and return it has a hash
-sub _get_data {
+sub _get_input_as_hash {
 	my ($self,$data) = @_;
 	$self->{__INPUT_DATA} = $data;
 	require UNIVERSAL;
@@ -942,7 +1032,14 @@ sub _get_data {
 	}
 	# otherwise, it's already a hash reference
     elsif (ref $data eq 'HASH') {
-		return %$data;	
+        # be careful to actually copy array references
+        my %copy = %$data;
+        for (grep { ref $data->{$_} eq 'ARRAY' } keys %$data) {
+            my @array_copy = @{ $data->{$_} };
+            $copy{$_} = \@array_copy;
+        }
+
+		return %copy;
     }
 	else {
 		die "Data::FormValidator->validate() or check() called with invalid input data structure.";
@@ -1012,13 +1109,18 @@ sub _add_constraints_from_map {
 					else {
 						$result{$key} = $new;
 					}
-					warn "constraint_regexp_map: $key matches\n" if $profile->{debug};
+					warn "$map_name: $key matches\n" if $profile->{debug};
 				}
 			}
 	}
 	return %result;
 }
 
+sub _bool_overload_based_on_success {
+    my $results = shift;
+    return $results->success()
+}
+
 # =head2 _check_constraints()
 #
 # $self->_check_constraints(
@@ -1048,9 +1150,11 @@ sub _check_constraints {
 		next unless exists $valid->{$field};
 
 		my $is_constraint_list = 1 if (ref $constraint_list eq 'ARRAY');
-		my $untaint_this =  ($untaint_all || $untaint_href->{$field} || 0);
+		my $untaint_this = ($untaint_all || $untaint_href->{$field} || 0);
 
 		my @invalid_list;
+        # used to insure we only bother recording each failed constraint once
+		my %constraints_seen;
 		foreach my $constraint_spec (_arrayify($constraint_list)) {
 
 			# set current constraint field for use by get_current_constraint_field
@@ -1060,28 +1164,33 @@ sub _check_constraints {
 			# from being accidently shared
 			$self->{__CURRENT_CONSTRAINT_NAME} = undef;
 
-			my $c = $self->_constraint_hash_build($constraint_spec,$untaint_this);
+			my $c = $self->_constraint_hash_build($constraint_spec,$untaint_this, $force_method_p);
 			$c->{is_method} = 1 if $force_method_p;
 
 			my $is_value_list = 1 if (ref $valid->{$field} eq 'ARRAY');
+            my %param_data = ( $self->_get_input_as_hash($self->get_input_data) , %$valid );
 			if ($is_value_list) {
 				foreach (my $i = 0; $i < scalar @{ $valid->{$field}} ; $i++) {
-					my @params = $self->_constraint_input_build($c,$valid->{$field}->[$i],$valid);
+                    if( !exists $constraints_seen{\$c} ) {
 
-					# set current constraint field for use by get_current_constraint_value
-					$self->{__CURRENT_CONSTRAINT_VALUE} = $valid->{$field}->[$i];
+                        my @params = $self->_constraint_input_build($c,$valid->{$field}->[$i],\%param_data);
 
-					my ($match,$failed) = $self->_constraint_check_match($c,\@params,$untaint_this);
-					if ($failed->{failed}) {
-						push @invalid_list, $failed;
-					}
-					else {
-						 $valid->{$field}->[$i] = $match if $untaint_this;
-					}
+                        # set current constraint field for use by get_current_constraint_value
+                        $self->{__CURRENT_CONSTRAINT_VALUE} = $valid->{$field}->[$i];
+
+                        my ($match,$failed) = $self->_constraint_check_match($c,\@params,$untaint_this);
+                        if ($failed->{failed}) {
+                            push @invalid_list, $failed;
+                            $constraints_seen{\$c} = 1;
+                        }
+                        else {
+                            $valid->{$field}->[$i] = $match if $untaint_this;
+                        }
+                    }
 				}
 			}
 			else {
-				my @params = $self->_constraint_input_build($c,$valid->{$field},$valid);
+				my @params = $self->_constraint_input_build($c,$valid->{$field},\%param_data);
 
 				# set current constraint field for use by get_current_constraint_value
 				$self->{__CURRENT_CONSTRAINT_VALUE} = $valid->{$field};
